//
//  JMActionRowTableViewController.h
//
//  Created by Jérôme Miglino on 7/18/12.
//
//  Copyright (c) 2012, Jerome Miglino. All rights reserved.
//
//  Permission is hereby granted, free of charge, to any person
//  obtaining a copy of this software and associated documentation
//  files (the "Software"), to deal in the Software without
//  restriction, including without limitation the rights to use,
//  copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following
//  conditions:
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.

#import "JMActionRowTableViewController.h"

@interface JMActionRowTableViewController ()

@end

#define kTableViewCellRegularHeight         60.0
#define kTableViewCellToolBarHeight         44.0
#define kTableViewCellRegularIdentifier     @"TableViewCellRegular"
#define kTableViewCellToolBarIdentifier     @"TableViewCellToolBar"

@implementation JMActionRowTableViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        //Initializing the data source
        _data = [NSArray arrayWithObjects:@"1",@"2",@"3",@"4",@"5",@"6",@"7",@"8",@"9",@"10",@"11",@"12",@"13",@"14",nil];
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    //Setting the delegate of the TableView
    [self.tableView setDelegate:self];
    //Don't display empty row
    [self.tableView setTableFooterView:[[UIView alloc] initWithFrame:CGRectZero]];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    //Update this according to your datasource.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section according to your datasource.
    NSInteger numberOfRow = [_data count];
    //If the ControlRow is displayed we need to return one more element
    return (_controlRowIndexPath) ? numberOfRow+1 : numberOfRow;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *cellIdentifier = (_controlRowIndexPath && [indexPath isEqual:_controlRowIndexPath]) ? kTableViewCellToolBarIdentifier : kTableViewCellRegularIdentifier;
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    //Create your custom cells based on the cellIdentifier
    //If you use NIBs you can also use the tableView registerNib:forCellReuseIdentifier: method to provide Automatic Loading
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier];
    }
    
    //Just for the example, if this is the toolbar we set its background to Red
    if ([cellIdentifier isEqualToString:kTableViewCellToolBarIdentifier]) {
        [cell.contentView setBackgroundColor:[UIColor redColor]];
    }
    else {
        //We use the method to avoid a NSRange Exception.
        indexPath = [self modelIndexPathforIndexPath:indexPath];
        [[cell textLabel] setText:[_data objectAtIndex:indexPath.row]];
    }
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //If we reselect the already selected cell
    if ([indexPath isEqual:_tappedIndexPath]) {
        [tableView deselectRowAtIndexPath:indexPath animated:YES];
    }
    
    //We "correct" the indexPath, because the toolbar is not in the DataSource.
    indexPath = [self modelIndexPathforIndexPath:indexPath];
    
    //If there is already a toolbar display on the tableView
    NSIndexPath *indexPathToDelete = _controlRowIndexPath;
    
    if ([indexPath isEqual:_tappedIndexPath]) {
        _tappedIndexPath = nil;
        _controlRowIndexPath = nil;
    }
    else {
        _tappedIndexPath = indexPath;
        _controlRowIndexPath = [NSIndexPath indexPathForRow:indexPath.row+1 inSection:indexPath.section];
    }
    
    [tableView beginUpdates];
    //if we need ro remove a previous displayed toolbar (tapped twice or tapped another cell)
    if(indexPathToDelete){
        [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPathToDelete] 
                              withRowAnimation:UITableViewRowAnimationAutomatic];
    }
    //Add the toolbar below the tapped row
    if(_controlRowIndexPath){
        [self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:_controlRowIndexPath] 
                              withRowAnimation:UITableViewRowAnimationAutomatic];
    }
    
    [tableView endUpdates];
    
}

- (NSIndexPath *)modelIndexPathforIndexPath:(NSIndexPath *)indexPath
{
    if(_controlRowIndexPath  && indexPath.row > _controlRowIndexPath.row) {
        return [NSIndexPath indexPathForRow:indexPath.row - 1 inSection:indexPath.section];
    }
    return indexPath;
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //If we select the toolbar, we force to select its "Parent" row, the one which can handle the action
    if ([indexPath isEqual:_controlRowIndexPath]) {
        return _tappedIndexPath;
    }
    return indexPath;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return ([indexPath isEqual:_controlRowIndexPath]) ? kTableViewCellToolBarHeight : kTableViewCellRegularHeight;
}

@end
